//----------------------------------------------------------------------------
//	File: Date.cpp
//
//	class: Date
//	methods:
//		Date::Date(short day, short month, short year)
//		bool Date::operator==(const Comparable &other)const
//		bool Date::operator<(const Comparable &other)const
//		void Date::setCurrentDate(void)
//		void Date::setDayOfWeek(void)
//		void Date::setDayOfYear(void)
//		short Date::countLeaps(short year)const
//		void Date::setDayOfMonth(short dayOfMonth)
//		void Date::setMonth(short month)
//		void Date::setYear(short year)
//----------------------------------------------------------------------------
#include "Date.h"

namespace PB_DATETIME
{
//----------------------------------------------------------------------------
//    method:		Date(short day, short month, short year) -- constructor
//
//    description:	creates a new Date object
// 
//    Calls:            setCurrentDate()
//			daysInMonth(month, year)
//			setDayOfYear()
//			setDayOfWeek()
// 
//    Called By:	ostream& operator<<(ostream& sout, const CComplex &c)
// 
//    Parameters:	short day -- day to set
//			short month -- month to set
//			short year -- month to set
//
//    History Log:
//			2/9/08  PB  completed version 1.0
// ---------------------------------------------------------------------------
	Date::Date(short day, short month, short year)
	{
		setCurrentDate();
		setYear(year);
		setMonth(month);
		setDayOfMonth(day);
	}

//----------------------------------------------------------------------------
//    Class:		Date
//    method:		operator==(const Comparable& other) const
//
//    description:	true if the two objects are exactly the same
// 
//    Parameters:	const Comparable &other  -- the other Date to compare
//
//    Called By:	main, >=, <=
// 
//    Returns:          true if the two objects are exactly the same;
//				false otherwise
//    History Log:
//			5/8/16  PB  completed version 1.1
//----------------------------------------------------------------------------
	bool Date::operator==(const Comparable &other) const
	{
		bool returnValue = false;
		try
		{
			const Date &otherDate = 
				dynamic_cast<const Date&>(other);
			if(m_year == otherDate.m_year && m_dayOfYear ==
				otherDate.m_dayOfYear)
				returnValue = true;
		}
		catch(bad_cast e)
		{ }
		return returnValue;
    }
//----------------------------------------------------------------------------
//    Class:		Date
//    method:		operator<(const Comparable& other) const
//
//    description:	true if the current object is before other
// 
//    Parameters:	const Comparable &other  -- the other Date to compare
//
//    Called By:	main, >, <=
//
//    Exceptions:	throws bad_cast if other is not a Date
// 
//    Returns:		true if the current object is before other;
//				false otherwise
//    History Log:
//			5/8/16  PB  completed version 1.1
// ---------------------------------------------------------------------------
	bool Date::operator<(const Comparable &other) const
	{
		bool returnValue = false;
		// your code goes here
		const Date &dateCompare = dynamic_cast<const Date&> (other);
		if (dateCompare.m_year > this->m_year)
		{
			returnValue = true;
		}
		else if (dateCompare.m_year == this->m_year && dateCompare.m_month > this->m_month)
		{
			returnValue = true;
		}
		else if (dateCompare.m_year == this->m_year && dateCompare.m_month == this->m_month && dateCompare.m_dayOfMonth > this->m_dayOfMonth)
		{
			returnValue = true;
		}
		return returnValue; 
	}

//----------------------------------------------------------------------------
//    Class:		Date
//    method:		setCurrentDate()
//
//    description:	sets the date from the system clock
// 
//    Called By:	constructor, mutators
//
//    History Log:
//			2/9/08  PB  completed version 1.0
// ---------------------------------------------------------------------------
	void Date::setCurrentDate(void)
	{
		time_t rawtime;
		tm *currentTimePtr;
		time(&rawtime);
		currentTimePtr = localtime(&rawtime);

		m_year = currentTimePtr->tm_year + 1900;
		m_month = currentTimePtr->tm_mon;
		m_dayOfMonth = currentTimePtr->tm_mday;
		setDayOfYear();
        	setDayOfWeek();
	}

//----------------------------------------------------------------------------
//    Class:	Date
//    method:		setDayOfWeek(void)
//
//    description:	validates and sets the proper day of the week
//
//    History Log:
//                           2/9/08  PB  completed version 1.0
// ---------------------------------------------------------------------------
	void Date::setDayOfWeek(void)
	{
		short weekday = 0;
		short adjustTime = 0;
		if (!isLeapYear(this->m_year))
		{
			adjustTime++;
		}
		weekday = countLeaps(this->m_year) + (this->m_year - LOWYEAR + adjustTime) + m_dayOfYear;
		weekday = weekday % DAYSINWEEK;
		m_dayOfWeek = weekday;
	}

//----------------------------------------------------------------------------
//    Class:		Date
//    method:		setDayOfYear(void)
//
//    description:	validates and sets the proper day of the year
//
//    History Log:
//			2/9/08  PB  completed version 1.0
// ---------------------------------------------------------------------------
	void Date::setDayOfYear(void)
	{
		short day = 0;
		for (short i = 0; i < this->m_month; ++i)
		{
			day = day + daysInMonth(i, this->m_year);
		}
		day = day + this->m_dayOfMonth;
		m_dayOfYear = day;
	}

//----------------------------------------------------------------------------
//    Class:			Date
//    method:			countLeaps(short year)const
//
//    description:	returns the number of leap years from 1760 to year
// 
//    Calls:            isLeapYear(short)
// 
//    Returns:          the number of leap years from 1760 to year 
//
//    History Log:
//                           2/9/08  PB  completed version 1.0
//----------------------------------------------------------------------------
	short Date::countLeaps(short year)const
	{ 
		short leaps = 0;
		for(short i = LOWYEAR; i < year; i++)
		if(isLeapYear(i))
			leaps++; 
		return leaps;
	}

//----------------------------------------------------------------------------
//    Class:		Date
//    method:		setDayOfMonth(short dayOfMonth)
//
//    description:	mutator for m_dayOfMonth
// 
//    Calls:            daysInMonth(short, short)
//			setDayOfYear()
//			setDayOfWeek()
// 
//    Parameters:	dayOfMonth --  day of month to set
//
//    History Log:
//			2/9/08  PB  completed version 1.0
//----------------------------------------------------------------------------
	void Date::setDayOfMonth(short dayOfMonth)
	{
        	if(dayOfMonth > 0
			&& dayOfMonth <= daysInMonth(m_month, m_year))
        	{
            		m_dayOfMonth = dayOfMonth;
            		setDayOfYear();  
            		setDayOfWeek();
        	}
	}
  
//----------------------------------------------------------------------------
//    Class:	Date
//    method:		setMonth(short dmonth)
//
//    description:	mutator for m_month
// 
//    Calls:            daysInMonth(short, short)
//			setDayOfYear()
//			setDayOfWeek()
// 
//    Parameters:	month --  month to set
//
//    History Log:
//			2/9/08  PB  completed version 1.0
//----------------------------------------------------------------------------
	void Date::setMonth(short month)
	{
		short monthtemp = month - 1;
		if (monthtemp >= 0 && monthtemp <= MONTHSINYEAR)
		{
			if (m_dayOfMonth <= daysInMonth(m_month, m_year))
			{
				m_month = monthtemp;
				setDayOfYear();
				setDayOfWeek();
			}
		}
     
	}
 
//----------------------------------------------------------------------------
//    Class:		Date
//    method:		setYear(short year)
//
//    description:	mutator for m_year
// 
//    Calls:            isLeapYear(short)
//			setDayOfYear()
//			setDayOfWeek()
// 
//    Parameters:	year --  year to set
//
//    History Log:
//			 2/9/08  PB  completed version 1.0
//----------------------------------------------------------------------------
	void Date::setYear(short year)
	{
		if (year > LOWYEAR && !isLeapYear(m_year))
		{
			m_year = year;
			setDayOfYear();
			setDayOfWeek();
		}
	}

//----------------------------------------------------------------------------
//    Class:		Date
//    method:		input(istream& sin)
//
//    description:	Take the input date with format dd/mm/yyyy
// 
//    Calls: 
//    Parameters:	istream& sin
//
//    History Log:
//			 10/5/18  Implement function
//----------------------------------------------------------------------------
	void Date:: input(istream& sin)
	{
		short day = -1;
		short month = -1;
		short year = -1;

		if (sin >> day)
		{
			if (sin.peek() == '/')
			{
				sin.get();
				sin >> month;
				if (sin.peek() == '/')
				{
					sin.get();
					sin >> year;
				}
			}
		}
		setCurrentDate();
		setYear(year);
		setMonth(month);
		setDayOfMonth(day);
	}

//----------------------------------------------------------------------------
//    method:		print(ostream& sout)const
//
//    description:	Print the date with format dd/mm/yyyy
// 
//    Calls: 
//    Parameters:	ostream& sout
//
//    History Log:
//			 10/5/18  Implement function
//----------------------------------------------------------------------------
	void Date::print(ostream& sout)const
	{
		sout << weekdayName(static_cast<int> (m_dayOfWeek));
		sout << ", "
			<< monthName(m_month)
			<< " " << m_dayOfMonth << ", " << m_year;
	}

//----------------------------------------------------------------------------
//    method:		tomorrow() const
//    description:	Get tomorrow day with current
// 
//    Calls: 
//    Parameters:	None
//
//    History Log:
//			 10/5/18  Implement function
//----------------------------------------------------------------------------
	Date Date::tomorrow() const
	{		
		Date nextDay = *this;
		if (m_dayOfMonth + 1 >= daysInMonth(m_month, m_year))
		{
			nextDay.setDayOfMonth(1);
			if (m_month == 11)
			{
				nextDay.setMonth(1);
				nextDay.setYear(m_year + 1);
			}
			else
				nextDay.setMonth(m_month + 1);
		}
		else
			nextDay.setDayOfMonth(m_dayOfMonth + 1);

		return nextDay;
	}

//----------------------------------------------------------------------------
//    method:		yesterday() const
//    description:	Get yesterday date with current
// 
//    Calls: 
//    Parameters:	None
//
//    History Log:
//			 10/5/18  Implement function
//----------------------------------------------------------------------------
	Date Date::yesterday() const
	{
		Date oneDayAgo = *this;

		if (m_dayOfMonth == 1)
		{
			if (m_month == 0)
			{
				oneDayAgo.setYear(m_year - 1);
				oneDayAgo.setMonth(MONTHSINYEAR);
				oneDayAgo.setDayOfMonth(daysInMonth(MONTHSINYEAR, m_year));
			}
			oneDayAgo.setMonth(m_month - 1);
			oneDayAgo.setDayOfMonth(daysInMonth(m_month - 1, m_year));
		}
		else
			oneDayAgo.setDayOfMonth(m_dayOfMonth - 1);

		return oneDayAgo;
	}

//----------------------------------------------------------------------------
//    method:		operator<<(ostream& sout, const Date& date)
//    description:	operator <<
// 
//    Calls: 
//    Parameters:	None
//
//    History Log:
//			 10/5/18  Implement function
//----------------------------------------------------------------------------
	ostream& operator<<(ostream& sout, const Date& date)
	{
		date.print(sout);
		return sout;
	}

//----------------------------------------------------------------------------
//    method:		operator<<(ostream& sout, const Date& date)
//    description:	operator <<
// 
//    Calls: 
//    Parameters:	None
//
//    History Log:
//			 10/5/18  Implement function
//----------------------------------------------------------------------------
	istream& operator>>(istream& sin, Date& date)
	{
		date.input(sin);
		return sin;
	}
}
